%macro BRR_CORR(	INFILE=,
			REPLI_ROOT=,
			BYVAR=,
			VAR1=,
			VAR2=,
			OUTFILE=);

/* 

MEANING OF THE MACRO ARGUMENTS

INFILE	= 	INPUT DATA FILE.
REPLI_ROOT	= 	ROOT OF THE FINAL WEIGHT AND 80 REPLICATES VARIABLE NAMES. 
			FINAL WEIGHT VARIABLE NAME MUST BE THE REPLICATION ROOT FOLLOWED BY 0.
BYVAR		= 	BREAKDOWN VARIABLES
VAR1		=	FIRST NUMERIC VARIABLE
VAR2		=	SECOND NUMERIC VARIABLE
OUTFILE	= 	FILE WITH THE STATISTIC ESTIMATES AND THEIR STANDARD ERROR ESTIMATE

*/ 

OPTIONS NONOTES;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP1;
RUN;

PROC SORT DATA=&INFILE OUT=BRRDATA (KEEP=&REPLI_ROOT.0-&REPLI_ROOT.80 &BYVAR &VAR1 &VAR2);
	BY &BYVAR;
RUN;

%DO I = 0 %TO 80;

	PROC CORR DATA=BRRDATA VARDEF=WGT NOPRINT OUTP=CORR_TEMP;
		VAR &VAR1 ;
		WITH &VAR2;
		BY &BYVAR;
		WEIGHT &REPLI_ROOT&I;
	RUN;

	DATA CORR_TEMP;
		SET CORR_TEMP;
		L=&I;
	RUN;

	PROC APPEND BASE = BRR_TEMP1 DATA=CORR_TEMP;
	RUN;
%END;

DATA BRR_TEMP1;
	SET BRR_TEMP1;
	IF (_TYPE_ NE "CORR") THEN DELETE;
	PV=&VAR1;
	KEEP &BYVAR L PV;
RUN;

DATA BRR_TEMP2(KEEP=&BYVAR PV) BRR_TEMP3(KEEP=&BYVAR STAT);
	SET BRR_TEMP1;
	IF L > 0 THEN OUTPUT BRR_TEMP2;     
	ELSE DO;
		STAT =PV;
		OUTPUT BRR_TEMP3;
	END;
RUN;


PROC SORT DATA=BRR_TEMP2;
	BY &BYVAR;
RUN;

PROC SORT DATA=BRR_TEMP3;
	BY &BYVAR;
RUN;

DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY &BYVAR;
	VARI=((PV-STAT)**2)*(1/20);
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
	VAR VARI;
	BY &BYVAR;
	OUTPUT OUT=BRR_TEMP5 SUM=SS;
RUN;

OPTIONS NOTES;

DATA &OUTFILE;
	MERGE BRR_TEMP3 BRR_TEMP5;
	BY &BYVAR;
	SESTAT=(SS)**0.5;
	KEEP &BYVAR STAT SESTAT;
RUN;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 CORR_TEMP BRRDATA;
RUN;

%MEND BRR_CORR;


