%macro BRR_FREQ(	INFILE=,
			REPLI_ROOT=,
			BYVAR=,
			VAR=,
			OUTFILE=);

/* 

MEANING OF THE MACRO ARGUMENTS

INFILE		= 	INPUT DATA FILE.
REPLI_ROOT	= 	ROOT OF THE FINAL WEIGHT AND 80 REPLICATES VARIABLE NAMES. 
				FINAL WEIGHT VARIABLE NAME MUST BE THE REPLICATION ROOT FOLLOWED BY 0.
BYVAR		= 	BREAKDOWN VARIABLES
VAR			= 	VARIABLES ON WHICH PERCENTAGES WILL BE COMPUTED
OUTFILE		= 	FILE WITH THE STATISTIC ESTIMATES AND THEIR STANDARD ERROR ESTIMATE

*/ 

OPTIONS NONOTES;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP0;
RUN;

PROC SORT DATA=&INFILE
	OUT=BRRDATA(KEEP=&REPLI_ROOT.0-&REPLI_ROOT.80 &BYVAR &VAR);
	BY &BYVAR &VAR;
RUN;

%DO I = 0 %TO 80;
	PROC FREQ DATA=BRRDATA NOPRINT;
		TABLE &VAR /OUT=FREQ_TEMP ;
		BY &BYVAR;
		WEIGHT &REPLI_ROOT&I;
	RUN;

	DATA FREQ_TEMP;
		SET FREQ_TEMP;
		L=&I;
		KEEP &BYVAR L &VAR PERCENT;
	RUN;

	PROC APPEND BASE = BRR_TEMP0 DATA=FREQ_TEMP;
	RUN;
%END;

PROC SORT DATA=BRR_TEMP0 OUT=BRR_TEMP1 (RENAME=(PERCENT=PV));
	BY &BYVAR L &VAR;
RUN;

DATA BRR_TEMP2(KEEP=&BYVAR &VAR PV)BRR_TEMP3(KEEP=&BYVAR &VAR STAT);
	SET BRR_TEMP1;
	IF L > 0 THEN OUTPUT BRR_TEMP2;     
	ELSE DO;
		STAT =PV;
		OUTPUT BRR_TEMP3;
	END;
RUN;

PROC SORT DATA=BRR_TEMP2;
	BY &BYVAR &VAR;
RUN;

PROC SORT DATA=BRR_TEMP3;
	BY &BYVAR &VAR;
RUN;

DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY &BYVAR &VAR;
	VARI=((PV-STAT)**2)*(1/20);
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
	VAR VARI;
	BY &BYVAR &VAR;
	OUTPUT OUT=BRR_TEMP5 SUM=SS;
RUN;

OPTIONS NOTES;

DATA &OUTFILE;
	MERGE BRR_TEMP3 BRR_TEMP5;
	BY &BYVAR &VAR;
	SESTAT=(SS)**0.5;
	KEEP &BYVAR &VAR STAT SESTAT;
RUN;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP0 BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 FREQ_TEMP BRRDATA;
RUN;

%MEND BRR_FREQ;


