%macro BRR_REG_PV(INFILE=,
			REPLI_ROOT=,
			EXPLICA=,
			BYVAR=,
			PV_ROOT=,
			OUTFILE=);

/*

MEANING OF THE MACRO ARGUMENTS

INFILE	= 	INPUT DATA FILE.
REPLI_ROOT	= 	ROOT OF THE FINAL WEIGHT AND 80 REPLICATES VARIABLE NAMES. 
			FINAL WEIGHT VARIABLE NAME MUST BE THE REPLICATION ROOT FOLLOWED BY 0.
EXPLICA	=	LIST OF INDEPENDENT VARIABLES
BYVAR		= 	BREAKDOWN VARIABLES
PV_ROOT	= 	ROOT OF THE 5 PLAUSIBLE VALUES VARIABLES NAMES
OUTFILE	=	FILE WITH THE STATISTIC ESTIMATES AND THEIR STANDARD ERROR ESTIMATE

*/

OPTIONS NONOTES;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP1;
RUN;

PROC SORT DATA=&INFILE OUT=BRRDATA (KEEP=&REPLI_ROOT.0-&REPLI_ROOT.80 &BYVAR &PV_ROOT.1-&PV_ROOT.5 &EXPLICA);
	BY &BYVAR;
RUN;

%DO I = 0 %TO 80;
	PROC REG DATA=BRRDATA OUTEST=COEF_TEMP NOPRINT;
		MODEL &PV_ROOT.1=&EXPLICA;
		MODEL &PV_ROOT.2=&EXPLICA;
		MODEL &PV_ROOT.3=&EXPLICA;
		MODEL &PV_ROOT.4=&EXPLICA;
		MODEL &PV_ROOT.5=&EXPLICA;
		WEIGHT &REPLI_ROOT&I;
		BY &BYVAR;
	RUN;

	DATA COEF_TEMP;
		SET COEF_TEMP;
		L=&I;
	RUN;

 	PROC APPEND BASE = BRR_TEMP1 DATA=COEF_TEMP;
 	RUN;

%END;
PROC SORT DATA=BRR_TEMP1;
	BY &BYVAR L;
RUN;

PROC TRANSPOSE DATA=BRR_TEMP1 OUT=BRR_TEMP PREFIX=PV;
	BY &BYVAR L;
	VAR INTERCEPT &EXPLICA;
RUN; 

DATA BRR_TEMP1 ;
	SET BRR_TEMP (RENAME=(_NAME_=CLASS));
	DROP _LABEL_;
RUN;

%LET I=1;
%DO %WHILE(%LENGTH(%SCAN(&EXPLICA,&I)));
	%LET I=%EVAL(&I+1);
%END;
%let NB=%EVAL(&I-1);

%DO J=0 %to &NB;
	%IF &J=0 %THEN %DO;
        %LET INDEP=Intercept;
	%END;
	%IF &J>0 %THEN %DO;
		%LET INDEP=%SCAN(&EXPLICA,&J);
	%END;

   	DATA BRR_TEMP1;
			SET BRR_TEMP1;
			IF (UPCASE(CLASS)=UPCASE("&INDEP")) THEN ORDRE=&J;
	RUN;
%END;

DATA BRR_TEMP2(KEEP=&BYVAR CLASS ORDRE PV1-PV5)	BRR_TEMP3(KEEP=&BYVAR CLASS ORDRE FIN1-FIN5 STAT MESVAR);
	SET BRR_TEMP1;
	IF L > 0 THEN OUTPUT BRR_TEMP2;     
	ELSE DO;
		STAT =(PV1+PV2+PV3+PV4+PV5)/5;
		FIN1=PV1;
		FIN2=PV2;
		FIN3=PV3;
		FIN4=PV4;
		FIN5=PV5;
		MESVAR=(((STAT-FIN1)**2)+((STAT-FIN2)**2)+((STAT-FIN3)**2)+((STAT-FIN4)**2)+((STAT-FIN5)**2))/4;
		OUTPUT BRR_TEMP3;
	END;
RUN;

PROC SORT DATA=BRR_TEMP2;
	BY &BYVAR CLASS;
RUN;

PROC SORT DATA=BRR_TEMP3;
	BY &BYVAR CLASS;
RUN;

DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY &BYVAR CLASS;
	ARRAY A (5)
		PV1-PV5;
	ARRAY B (5)
		FIN1-FIN5;
	ARRAY C(5)
		VAR1-VAR5;
	DO I=1 TO 5;
		C(I)=(1/20)*((A(I)-B(I))**2);
	END;
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
	VAR VAR1 VAR2 VAR3 VAR4 VAR5;
	BY &BYVAR CLASS;
	OUTPUT OUT=BRR_TEMP5 SUM=SS1 SS2 SS3 SS4 SS5;
RUN;

DATA BRR_TEMP6;
	MERGE BRR_TEMP3 BRR_TEMP5;
	BY &BYVAR CLASS;
	SAMP=(SS1+SS2+SS3+SS4+SS5)/5;
	FINVAR=(SAMP+(1.2*MESVAR));
	SESTAT=(FINVAR)**0.5;
	KEEP &BYVAR CLASS ORDRE STAT SESTAT;
RUN;

OPTIONS NOTES;

PROC SORT DATA=BRR_TEMP6 OUT=&OUTFILE (DROP=ORDRE);
	BY &BYVAR ORDRE;
RUN;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 BRR_TEMP6 COEF_TEMP BRRDATA;
RUN;

%MEND BRR_REG_PV;

