%macro BRR_REG(	INFILE=,
			REPLI_ROOT=,
			VARDEP=,
			EXPLICA=,
			BYVAR=,
			OUTFILE=);

/*

MEANING OF THE MACRO ARGUMENTS

INFILE	= 	INPUT DATA FILE.
REPLI_ROOT	= 	ROOT OF THE FINAL WEIGHT AND 80 REPLICATES VARIABLE NAMES. 
			FINAL WEIGHT VARIABLE NAME MUST BE THE REPLICATION ROOT FOLLOWED BY 0.
VARDEP	= 	DEPENDENT VARIABLE 
EXPLICA	=	LIST OF INDEPENDENT VARIABLES
BYVAR		= 	BREAKDOWN VARIABLES
OUTFILE	=	FILE WITH THE STATISTIC ESTIMATES AND THEIR STANDARD ERROR ESTIMATE.

*/

OPTIONS NONOTES;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP1;
RUN;

PROC SORT DATA=&INFILE OUT=BRRDATA (KEEP=&REPLI_ROOT.0-&REPLI_ROOT.80 &BYVAR &VARDEP &EXPLICA);
	BY &BYVAR;
RUN;


%DO I = 0 %TO 80;

	PROC REG DATA=BRRDATA OUTEST=COEF_TEMP NOPRINT;
		MODEL &VARDEP=&EXPLICA;
		WEIGHT &REPLI_ROOT&I;
		BY &BYVAR;
	RUN;

	DATA COEF_TEMP;
		SET COEF_TEMP;
		L=&I;
	RUN;

	PROC APPEND BASE = BRR_TEMP1 DATA=COEF_TEMP;
	RUN;
%END;

PROC SORT DATA=BRR_TEMP1;
	BY &BYVAR L;
RUN;

PROC TRANSPOSE DATA=BRR_TEMP1 OUT=BRR_TEMP;
	BY &BYVAR L;
	VAR INTERCEPT &EXPLICA;
RUN; 

DATA BRR_TEMP1 ;
	SET BRR_TEMP (RENAME=(_NAME_=CLASS COL1=COEF));
RUN;

%LET I=1;
%DO %WHILE(%LENGTH(%SCAN(&EXPLICA,&I)));
	%LET I=%EVAL(&I+1);
%END;
%LET NB=%EVAL(&I-1);

%DO J=0 %to &NB;
	%IF &J=0 %THEN %DO;
		%LET INDEP=Intercept;
	%END;
	%IF &J>0 %THEN %DO;
		%LET INDEP=%SCAN(&EXPLICA,&J);
	%END;

	DATA BRR_TEMP1;
		SET BRR_TEMP1;
		IF (UPCASE(CLASS)=UPCASE("&INDEP")) THEN ORDRE=&J;
	RUN;
%END;

DATA BRR_TEMP2(KEEP=&BYVAR CLASS COEF) BRR_TEMP3(KEEP=&BYVAR CLASS ORDRE STAT);
	SET BRR_TEMP1;
	IF L > 0 THEN OUTPUT BRR_TEMP2;     
	ELSE DO;
		STAT =COEF;
		OUTPUT BRR_TEMP3;
	END;
RUN;

PROC SORT DATA=BRR_TEMP2;
	BY &BYVAR CLASS;
RUN;

PROC SORT DATA=BRR_TEMP3;
	BY &BYVAR CLASS;
RUN;

DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY &BYVAR CLASS;
	VARI=((COEF-STAT)**2)*(1/20);
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
	VAR VARI;
	BY &BYVAR CLASS;
	OUTPUT OUT=BRR_TEMP5 SUM=SS;
RUN;

DATA BRR_TEMP6;
	MERGE BRR_TEMP3 BRR_TEMP5;
	BY &BYVAR CLASS;
	SESTAT=(SS)**0.5;
	KEEP &BYVAR CLASS ORDRE STAT SESTAT;
RUN;

OPTIONS NOTES;

PROC SORT DATA=BRR_TEMP6 OUT=&OUTFILE (DROP=ORDRE);
	BY &BYVAR ORDRE;
RUN;
PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 BRR_TEMP6 COEF_TEMP BRRDATA;
RUN;

%MEND BRR_REG;


